@@ -52,6 +52,7 @@
 #include "BeagleWksDRParser.hxx"
 #include "BeagleWksSSParser.hxx"
 #include "CanvasParser.hxx"
+#include "Canvas5BMParser.hxx"
 #include "Canvas5Parser.hxx"
 #include "ClarisDrawParser.hxx"
 #include "ClarisWksParser.hxx"
@@ -533,10 +534,14 @@ std::shared_ptr<MWAWGraphicParser> getGraphicParserFromHeader(MWAWInputStreamPtr
         parser.reset(new BeagleWksDRParser(input, rsrcParser, header));
       break;
     case MWAWDocument::MWAW_T_CANVAS:
-      if (header->getMajorVersion()<=3)
-        parser.reset(new CanvasParser(input, rsrcParser, header));
-      else
-        parser.reset(new Canvas5Parser(input, rsrcParser, header));
+      if (header->getKind()==MWAWDocument::MWAW_K_DRAW) {
+        if (header->getMajorVersion()<=3)
+          parser.reset(new CanvasParser(input, rsrcParser, header));
+        else
+          parser.reset(new Canvas5Parser(input, rsrcParser, header));
+      }
+      else if (header->getMajorVersion()>=5)
+        parser.reset(new Canvas5BMParser(input, rsrcParser, header));
       break;
     case MWAWDocument::MWAW_T_DRAWINGTABLE:
       parser.reset(new DrawTableParser(input, rsrcParser, header));
