@@ -64,13 +64,22 @@ namespace MWAWGraphicListenerInternal
 /** the global graphic state of MWAWGraphicListener */
 struct GraphicState {
   //! constructor
-  explicit GraphicState(std::vector<MWAWPageSpan> const &pageList) :
-    m_pageList(pageList), m_metaData(),
-    m_isDocumentStarted(false), m_isPageSpanOpened(false), m_isMasterPageSpanOpened(false), m_isAtLeastOnePageOpened(false),
-    m_isHeaderFooterStarted(false), m_pageSpan(), m_sentListMarkers(), m_subDocuments(),
-    m_section()
+  explicit GraphicState(std::vector<MWAWPageSpan> const &pageList)
+    : m_pageList(pageList)
+    , m_metaData()
+    , m_isDocumentStarted(false)
+    , m_isPageSpanOpened(false)
+    , m_isMasterPageSpanOpened(false)
+    , m_isAtLeastOnePageOpened(false)
+    , m_isHeaderFooterStarted(false)
+    , m_pageSpan()
+    , m_sentListMarkers()
+    , m_subDocuments()
+    , m_section()
   {
   }
+  GraphicState &operator=(GraphicState const &)=default;
+  GraphicState &operator=(GraphicState &&)=default;
   //! destructor
   ~GraphicState()
   {
@@ -104,12 +113,14 @@ struct State {
   //! constructor
   State();
   //! destructor
-  ~State() { }
+  ~State()
+  {
+  }
 
-//! returns true if we are in a text zone, ie. either in a textbox or a table cell
+  //! returns true if we are in a text zone, ie. either in a textbox or a table cell
   bool isInTextZone() const
   {
-    return m_inNote || m_isTextBoxOpened || m_isTableCellOpened;
+    return m_inNote || m_inLink || m_isTextBoxOpened || m_isTableCellOpened;
   }
   //! the origin position
   MWAWVec2f m_origin;
@@ -121,7 +132,7 @@ struct State {
   //! the paragraph
   MWAWParagraph m_paragraph;
   //! the list of list
-  shared_ptr<MWAWList> m_list;
+  std::shared_ptr<MWAWList> m_list;
 
   //! a flag to know if openFrame was called
   bool m_isFrameOpened;
@@ -159,32 +170,59 @@ struct State {
   libmwaw::SubDocumentType m_subDocumentType;
 
 private:
-  State(const State &);
-  State &operator=(const State &);
+  State(const State &) = delete;
+  State &operator=(const State &) = delete;
 };
 
-State::State() : m_origin(0,0),
-  m_textBuffer(""), m_font(20,12)/* default time 12 */, m_paragraph(), m_list(),
-  m_isFrameOpened(false), m_framePosition(), m_frameStyle(), m_isTextBoxOpened(false),
-  m_isGroupOpened(false), m_isLayerOpened(false),
-  m_isSpanOpened(false), m_isParagraphOpened(false), m_isListElementOpened(false),
-  m_firstParagraphInPageSpan(true), m_listOrderedLevels(),
-  m_isTableOpened(false), m_isTableRowOpened(false), m_isTableColumnOpened(false),
-  m_isTableCellOpened(false),
-  m_currentPage(0), m_numPagesRemainingInSpan(0), m_currentPageNumber(1),
-  m_inLink(false), m_inNote(false), m_inSubDocument(false), m_subDocumentType(libmwaw::DOC_NONE)
+State::State()
+  : m_origin(0,0)
+  , m_textBuffer("")
+  , m_font(20,12)/* default time 12 */
+  , m_paragraph()
+  , m_list()
+  , m_isFrameOpened(false)
+  , m_framePosition()
+  , m_frameStyle()
+  , m_isTextBoxOpened(false)
+  , m_isGroupOpened(false)
+  , m_isLayerOpened(false)
+  , m_isSpanOpened(false)
+  , m_isParagraphOpened(false)
+  , m_isListElementOpened(false)
+  , m_firstParagraphInPageSpan(true)
+  , m_listOrderedLevels()
+  , m_isTableOpened(false)
+  , m_isTableRowOpened(false)
+  , m_isTableColumnOpened(false)
+  , m_isTableCellOpened(false)
+  , m_currentPage(0)
+  , m_numPagesRemainingInSpan(0)
+  , m_currentPageNumber(1)
+  , m_inLink(false)
+  , m_inNote(false)
+  , m_inSubDocument(false)
+  , m_subDocumentType(libmwaw::DOC_NONE)
 {
 }
 }
 
-MWAWGraphicListener::MWAWGraphicListener(MWAWParserState &parserState, std::vector<MWAWPageSpan> const &pageList, librevenge::RVNGDrawingInterface *documentInterface) : MWAWListener(),
-  m_ds(new MWAWGraphicListenerInternal::GraphicState(pageList)), m_ps(new MWAWGraphicListenerInternal::State),
-  m_psStack(), m_parserState(parserState), m_documentInterface(documentInterface)
+MWAWGraphicListener::MWAWGraphicListener(MWAWParserState &parserState, std::vector<MWAWPageSpan> const &pageList, librevenge::RVNGDrawingInterface *documentInterface)
+  : MWAWListener()
+  , m_ds(new MWAWGraphicListenerInternal::GraphicState(pageList))
+  , m_ps(new MWAWGraphicListenerInternal::State)
+  , m_psStack()
+  , m_parserState(parserState)
+  , m_documentInterface(documentInterface)
 {
 }
 
-MWAWGraphicListener::MWAWGraphicListener(MWAWParserState &parserState, MWAWBox2f const &box, librevenge::RVNGDrawingInterface *documentInterface) : MWAWListener(),
-  m_ds(), m_ps(new MWAWGraphicListenerInternal::State), m_psStack(), m_parserState(parserState), m_documentInterface(documentInterface)
+MWAWGraphicListener::MWAWGraphicListener(MWAWParserState &parserState, MWAWBox2f const &box, librevenge::RVNGDrawingInterface *documentInterface)
+  : MWAWListener()
+  , m_ds()
+  , m_ps(new MWAWGraphicListenerInternal::State)
+  , m_psStack()
+  , m_parserState(parserState)
+  , m_documentInterface(documentInterface)
 {
   MWAWPageSpan pageSpan;
   pageSpan.setMargins(0);
@@ -412,7 +450,7 @@ void MWAWGraphicListener::openLink(MWAWLink const &link)
   m_documentInterface->openLink(propList);
   _pushParsingState();
   m_ps->m_inLink=true;
-// we do not want any close open paragraph in a link
+  // we do not want any close open paragraph in a link
   m_ps->m_isParagraphOpened=true;
 }
 
@@ -422,6 +460,7 @@ void MWAWGraphicListener::closeLink()
     MWAW_DEBUG_MSG(("MWAWGraphicListener::closeLink: closed outside a link\n"));
     return;
   }
+  _flushText();
   m_documentInterface->closeLink();
   _popParsingState();
 }
@@ -535,7 +574,7 @@ void MWAWGraphicListener::_openPageSpan(bool sendHeaderFooters)
   }
   m_ds->m_isAtLeastOnePageOpened=true;
   unsigned actPage = 0;
-  std::vector<MWAWPageSpan>::iterator it = m_ds->m_pageList.begin();
+  auto it = m_ds->m_pageList.begin();
   m_ps->m_currentPage++;
   while (true) {
     actPage+=static_cast<unsigned>(it->getPageSpan());
@@ -704,7 +743,7 @@ void MWAWGraphicListener::_closeListElement()
 
 int MWAWGraphicListener::_getListId() const
 {
-  size_t newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex.get());
+  auto newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex.get());
   if (newLevel == 0) return -1;
   int newListId = m_ps->m_paragraph.m_listId.get();
   if (newListId > 0) return newListId;
@@ -713,8 +752,7 @@ int MWAWGraphicListener::_getListId() const
     MWAW_DEBUG_MSG(("MWAWGraphicListener::_getListId: the list id is not set, try to find a new one\n"));
     first = false;
   }
-  shared_ptr<MWAWList> list=m_parserState.m_listManager->getNewList
-                            (m_ps->m_list, int(newLevel), *m_ps->m_paragraph.m_listLevel);
+  auto list=m_parserState.m_listManager->getNewList(m_ps->m_list, int(newLevel), *m_ps->m_paragraph.m_listLevel);
   if (!list) return -1;
   return list->getId();
 }
@@ -729,7 +767,7 @@ void MWAWGraphicListener::_changeList()
     _closeParagraph();
 
   size_t actualLevel = m_ps->m_listOrderedLevels.size();
-  size_t newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex.get());
+  auto newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex.get());
   if (newLevel>100) {
     MWAW_DEBUG_MSG(("MWAWGraphicListener::_changeList: find level=%d, set it to 100\n", static_cast<int>(newLevel)));
     newLevel=100;
@@ -746,7 +784,7 @@ void MWAWGraphicListener::_changeList()
   }
 
   if (newLevel) {
-    shared_ptr<MWAWList> theList;
+    std::shared_ptr<MWAWList> theList;
 
     theList=m_parserState.m_listManager->getList(newListId);
     if (!theList) {
@@ -784,6 +822,8 @@ void MWAWGraphicListener::_openSpan()
 {
   if (m_ps->m_isTableOpened && !m_ps->m_isTableCellOpened)
     return;
+  if (m_ps->m_inLink)
+    return;
   if (!m_ps->isInTextZone()) {
     MWAW_DEBUG_MSG(("MWAWGraphicListener::_openSpan: called outsize a text zone\n"));
     return;
@@ -862,7 +902,7 @@ bool MWAWGraphicListener::isHeaderFooterOpened() const
   return m_ds->m_isHeaderFooterStarted;
 }
 
-bool MWAWGraphicListener::insertHeader(MWAWSubDocumentPtr subDocument, librevenge::RVNGPropertyList const &extras)
+bool MWAWGraphicListener::insertHeader(MWAWSubDocumentPtr const &subDocument, librevenge::RVNGPropertyList const &extras)
 {
   if (m_ds->m_isHeaderFooterStarted) {
     MWAW_DEBUG_MSG(("MWAWGraphicListener::insertHeader: Oops a header/footer is already opened\n"));
@@ -883,7 +923,7 @@ bool MWAWGraphicListener::insertHeader(MWAWSubDocumentPtr subDocument, libreveng
   return true;
 }
 
-bool MWAWGraphicListener::insertFooter(MWAWSubDocumentPtr subDocument, librevenge::RVNGPropertyList const &extras)
+bool MWAWGraphicListener::insertFooter(MWAWSubDocumentPtr const &subDocument, librevenge::RVNGPropertyList const &extras)
 {
   if (m_ds->m_isHeaderFooterStarted) {
     MWAW_DEBUG_MSG(("MWAWGraphicListener::insertFooter: Oops a header/footer is already opened\n"));
@@ -1071,7 +1111,7 @@ void MWAWGraphicListener::insertPicture(MWAWPosition const &pos, MWAWEmbeddedObj
 }
 
 void MWAWGraphicListener::insertTextBox
-(MWAWPosition const &pos, MWAWSubDocumentPtr subDocument, MWAWGraphicStyle const &style)
+(MWAWPosition const &pos, MWAWSubDocumentPtr const &subDocument, MWAWGraphicStyle const &style)
 {
   if (!m_ds->m_isDocumentStarted) {
     MWAW_DEBUG_MSG(("MWAWGraphicListener::insertTextBox: the document is not started\n"));
@@ -1107,7 +1147,7 @@ void MWAWGraphicListener::insertTextBox
   closeFrame();
 }
 
-void MWAWGraphicListener::insertGroup(MWAWBox2f const &bdbox, MWAWSubDocumentPtr subDocument)
+void MWAWGraphicListener::insertGroup(MWAWBox2f const &bdbox, MWAWSubDocumentPtr const &subDocument)
 {
   if (!m_ds->m_isDocumentStarted || m_ps->isInTextZone()) {
     MWAW_DEBUG_MSG(("MWAWGraphicListener::insertGroup: can not insert a group\n"));
@@ -1134,7 +1174,7 @@ void MWAWGraphicListener::insertTable
   _startSubDocument();
   m_ps->m_subDocumentType = libmwaw::DOC_TABLE;
 
-  shared_ptr<MWAWListener> listen(this, MWAW_shared_ptr_noop_deleter<MWAWGraphicListener>());
+  std::shared_ptr<MWAWListener> listen(this, MWAW_shared_ptr_noop_deleter<MWAWGraphicListener>());
   try {
     table.sendTable(listen);
   }
@@ -1564,7 +1604,7 @@ void MWAWGraphicListener::_handleFrameParameters(librevenge::RVNGPropertyList &l
 ///////////////////
 // subdocument
 ///////////////////
-void MWAWGraphicListener::handleSubDocument(MWAWVec2f const &orig, MWAWSubDocumentPtr subDocument, libmwaw::SubDocumentType subDocumentType)
+void MWAWGraphicListener::handleSubDocument(MWAWVec2f const &orig, MWAWSubDocumentPtr const &subDocument, libmwaw::SubDocumentType subDocumentType)
 {
   if (!m_ds->m_isDocumentStarted) {
     MWAW_DEBUG_MSG(("MWAWGraphicListener::handleSubDocument: the graphic is not started\n"));
@@ -1589,10 +1629,12 @@ void MWAWGraphicListener::handleSubDocument(MWAWVec2f const &orig, MWAWSubDocume
     m_ps->m_inNote=true;
   // Check whether the document is calling itself
   bool sendDoc = true;
-  for (size_t i = 0; i < m_ds->m_subDocuments.size(); i++) {
+  for (auto doc : m_ds->m_subDocuments) {
+    if (!doc)
+      continue;
     if (!subDocument)
       break;
-    if (subDocument == m_ds->m_subDocuments[i]) {
+    if (*subDocument == *doc) {
       MWAW_DEBUG_MSG(("MWAWGraphicListener::handleSubDocument: recursif call, stop...\n"));
       sendDoc = false;
       break;
@@ -1601,7 +1643,7 @@ void MWAWGraphicListener::handleSubDocument(MWAWVec2f const &orig, MWAWSubDocume
   if (sendDoc) {
     if (subDocument) {
       m_ds->m_subDocuments.push_back(subDocument);
-      shared_ptr<MWAWListener> listen(this, MWAW_shared_ptr_noop_deleter<MWAWListener>());
+      std::shared_ptr<MWAWListener> listen(this, MWAW_shared_ptr_noop_deleter<MWAWListener>());
       try {
         subDocument->parse(listen, subDocumentType);
       }
@@ -1651,9 +1693,9 @@ void MWAWGraphicListener::_endSubDocument()
 ///////////////////
 
 // ---------- state stack ------------------
-shared_ptr<MWAWGraphicListenerInternal::State> MWAWGraphicListener::_pushParsingState()
+std::shared_ptr<MWAWGraphicListenerInternal::State> MWAWGraphicListener::_pushParsingState()
 {
-  shared_ptr<MWAWGraphicListenerInternal::State> actual = m_ps;
+  auto actual = m_ps;
   m_psStack.push_back(actual);
   m_ps.reset(new MWAWGraphicListenerInternal::State);
   return actual;
