@@ -162,6 +162,108 @@ bool MWAWGraphicStyle::Pattern::getBinary(MWAWEmbeddedObject &picture) const
   return bitmap.getBinary(picture);
 }
 
+////////////////////////////////////////////////////////////
+// gradient
+////////////////////////////////////////////////////////////
+bool MWAWGraphicStyle::Gradient::getAverageColor(MWAWColor &color) const
+{
+  if (m_stopList.empty())
+    return false;
+  if (m_stopList.size()==1) {
+    color=m_stopList[0].m_color;
+    return true;
+  }
+  // fixme: check the offset are sorted and use then to compute a better barycenter
+  unsigned n[]= {0,0,0,0};
+  for (auto const &st : m_stopList) {
+    n[0]+=st.m_color.getRed();
+    n[1]+=st.m_color.getGreen();
+    n[2]+=st.m_color.getBlue();
+    n[3]+=st.m_color.getAlpha();
+  }
+  color=MWAWColor((unsigned char)(n[0]/(unsigned)(m_stopList.size())),
+                  (unsigned char)(n[1]/(unsigned)(m_stopList.size())),
+                  (unsigned char)(n[2]/(unsigned)(m_stopList.size())),
+                  (unsigned char)(n[3]/(unsigned)(m_stopList.size())));
+  return true;
+}
+
+void MWAWGraphicStyle::Gradient::addTo(librevenge::RVNGPropertyList &propList) const
+{
+  if (!hasGradient()) return;
+  propList.insert("draw:fill", "gradient");
+  switch (m_type) {
+  case G_Axial:
+    propList.insert("draw:style", "axial");
+    break;
+  case G_Radial:
+    propList.insert("draw:style", "radial");
+    break;
+  case G_Rectangular:
+    propList.insert("draw:style", "rectangular");
+    break;
+  case G_Square:
+    propList.insert("draw:style", "square");
+    break;
+  case G_Ellipsoid:
+    propList.insert("draw:style", "ellipsoid");
+    break;
+  case G_Linear:
+  case G_None:
+#if !defined(__clang__)
+  default:
+#endif
+    propList.insert("draw:style", "linear");
+    break;
+  }
+  if (m_stopList.size()==2 &&m_stopList[0].m_offset <= 0 &&
+      m_stopList[1].m_offset >=1) {
+    size_t first=(m_type==G_Linear || m_type==G_Axial) ? 0 : 1;
+    propList.insert("draw:start-color",m_stopList[first].m_color.str().c_str());
+    propList.insert("librevenge:start-opacity", double(m_stopList[first].m_opacity), librevenge::RVNG_PERCENT);
+    propList.insert("draw:end-color",m_stopList[1-first].m_color.str().c_str());
+    propList.insert("librevenge:end-opacity", double(m_stopList[1-first].m_opacity), librevenge::RVNG_PERCENT);
+  }
+  else {
+    librevenge::RVNGPropertyListVector gradient;
+    for (auto const &gr :m_stopList) {
+      librevenge::RVNGPropertyList grad;
+      grad.insert("svg:offset", double(gr.m_offset), librevenge::RVNG_PERCENT);
+      grad.insert("svg:stop-color", gr.m_color.str().c_str());
+      grad.insert("svg:stop-opacity", double(gr.m_opacity), librevenge::RVNG_PERCENT);
+      gradient.append(grad);
+    }
+    propList.insert("svg:linearGradient", gradient);
+  }
+  propList.insert("draw:angle", double(m_angle), librevenge::RVNG_GENERIC);
+  propList.insert("draw:border", double(m_border), librevenge::RVNG_PERCENT);
+  if (m_type != G_Linear) {
+    propList.insert("svg:cx", double(m_percentCenter[0]), librevenge::RVNG_PERCENT);
+    propList.insert("svg:cy", double(m_percentCenter[1]), librevenge::RVNG_PERCENT);
+  }
+  if (m_type == G_Radial)
+    propList.insert("svg:r", double(m_radius), librevenge::RVNG_PERCENT); // checkme
+}
+
+////////////////////////////////////////////////////////////
+// hatch
+////////////////////////////////////////////////////////////
+void MWAWGraphicStyle::Hatch::addTo(librevenge::RVNGPropertyList &propList) const
+{
+  if (!hasHatch()) return;
+  propList.insert("draw:fill", "hatch");
+  if (int(m_type)>=1 && int(m_type)<4) {
+    char const *wh[]= {"single", "double", "triple"};
+    propList.insert("draw:style", wh[int(m_type)-1]);
+  }
+  else {
+    MWAW_DEBUG_MSG(("MWAWGraphicStyle::Hatch::addTo: unknown hash type %d\n", int(m_type)));
+  }
+  propList.insert("draw:color", m_color.str().c_str());
+  propList.insert("draw:distance", double(m_distance), librevenge::RVNG_INCH);
+  if (m_rotation<0 || m_rotation>0) propList.insert("draw:rotation", double(m_rotation), librevenge::RVNG_GENERIC);
+}
+
 ////////////////////////////////////////////////////////////
 // style
 ////////////////////////////////////////////////////////////
@@ -270,64 +372,22 @@ void MWAWGraphicStyle::addTo(librevenge::RVNGPropertyList &list, bool only1D) co
     list.insert("draw:shadow-offset-x", double(m_shadowOffset[0])/72.*2.54, librevenge::RVNG_GENERIC); // cm
     list.insert("draw:shadow-offset-y", double(m_shadowOffset[1])/72.*2.54, librevenge::RVNG_GENERIC); // cm
   }
+  if (m_doNotPrint)
+    list.insert("style:print-content", false);
   if (only1D || !hasSurface()) {
     list.insert("draw:fill", "none");
     return;
   }
   list.insert("svg:fill-rule", m_fillRuleEvenOdd ? "evenodd" : "nonzero");
-  if (hasGradient()) {
-    list.insert("draw:fill", "gradient");
-    switch (m_gradientType) {
-    case G_Axial:
-      list.insert("draw:style", "axial");
-      break;
-    case G_Radial:
-      list.insert("draw:style", "radial");
-      break;
-    case G_Rectangular:
-      list.insert("draw:style", "rectangular");
-      break;
-    case G_Square:
-      list.insert("draw:style", "square");
-      break;
-    case G_Ellipsoid:
-      list.insert("draw:style", "ellipsoid");
-      break;
-    case G_Linear:
-    case G_None:
-#if !defined(__clang__)
-    default:
-#endif
-      list.insert("draw:style", "linear");
-      break;
+  if (hasGradient())
+    m_gradient.addTo(list);
+  else if (hasHatch()) {
+    m_hatch.addTo(list);
+    if (hasSurfaceColor()) {
+      list.insert("draw:fill-color", m_surfaceColor.str().c_str());
+      list.insert("draw:opacity", double(m_surfaceOpacity), librevenge::RVNG_PERCENT);
+      list.insert("draw:fill-hatch-solid", true);
     }
-    if (m_gradientStopList.size()==2 && m_gradientStopList[0].m_offset <= 0 &&
-        m_gradientStopList[1].m_offset >=1) {
-      size_t first=(m_gradientType==G_Linear || m_gradientType==G_Axial) ? 0 : 1;
-      list.insert("draw:start-color", m_gradientStopList[first].m_color.str().c_str());
-      list.insert("librevenge:start-opacity", double(m_gradientStopList[first].m_opacity), librevenge::RVNG_PERCENT);
-      list.insert("draw:end-color", m_gradientStopList[1-first].m_color.str().c_str());
-      list.insert("librevenge:end-opacity", double(m_gradientStopList[1-first].m_opacity), librevenge::RVNG_PERCENT);
-    }
-    else {
-      librevenge::RVNGPropertyListVector gradient;
-      for (auto const &gr : m_gradientStopList) {
-        librevenge::RVNGPropertyList grad;
-        grad.insert("svg:offset", double(gr.m_offset), librevenge::RVNG_PERCENT);
-        grad.insert("svg:stop-color", gr.m_color.str().c_str());
-        grad.insert("svg:stop-opacity", double(gr.m_opacity), librevenge::RVNG_PERCENT);
-        gradient.append(grad);
-      }
-      list.insert("svg:linearGradient", gradient);
-    }
-    list.insert("draw:angle", double(m_gradientAngle), librevenge::RVNG_GENERIC);
-    list.insert("draw:border", double(m_gradientBorder), librevenge::RVNG_PERCENT);
-    if (m_gradientType != G_Linear) {
-      list.insert("svg:cx", double(m_gradientPercentCenter[0]), librevenge::RVNG_PERCENT);
-      list.insert("svg:cy", double(m_gradientPercentCenter[1]), librevenge::RVNG_PERCENT);
-    }
-    if (m_gradientType == G_Radial)
-      list.insert("svg:r", double(m_gradientRadius), librevenge::RVNG_PERCENT); // checkme
   }
   else {
     bool done = false;
@@ -457,20 +517,9 @@ int MWAWGraphicStyle::cmp(MWAWGraphicStyle const &a) const
 
   diff = m_pattern.cmp(a.m_pattern);
   if (diff) return diff;
-
-  if (m_gradientType < a.m_gradientType) return -1;
-  if (m_gradientType > a.m_gradientType) return 1;
-  if (m_gradientAngle < a.m_gradientAngle) return -1;
-  if (m_gradientAngle > a.m_gradientAngle) return 1;
-  if (m_gradientStopList.size() < a.m_gradientStopList.size()) return 1;
-  if (m_gradientStopList.size() > a.m_gradientStopList.size()) return -1;
-  for (auto c : m_gradientStopList) {
-    diff = c.cmp(c);
-    if (diff) return diff;
-  }
-  if (m_gradientBorder < a.m_gradientBorder) return -1;
-  if (m_gradientBorder > a.m_gradientBorder) return 1;
-  diff=m_gradientPercentCenter.cmp(a.m_gradientPercentCenter);
+  diff = m_gradient.cmp(a.m_gradient);
+  if (diff) return diff;
+  diff = m_hatch.cmp(a.m_hatch);
   if (diff) return diff;
 
   size_t numBorders=m_bordersList.size();
@@ -492,8 +541,6 @@ int MWAWGraphicStyle::cmp(MWAWGraphicStyle const &a) const
   if (m_frameNextName < a.m_frameNextName) return -1;
   if (m_frameNextName > a.m_frameNextName) return 1;
 
-  if (m_gradientRadius < a.m_gradientRadius) return -1;
-  if (m_gradientRadius > a.m_gradientRadius) return 1;
   if (m_rotate < a.m_rotate) return -1;
   if (m_rotate > a.m_rotate) return 1;
   return 0;
@@ -559,45 +606,10 @@ std::ostream &operator<<(std::ostream &o, MWAWGraphicStyle const &st)
   }
   if (st.hasPattern())
     o << "pattern=[" << st.m_pattern << "],";
-  if (st.hasGradient()) {
-    o << "grad=[";
-    switch (st.m_gradientType) {
-    case MWAWGraphicStyle::G_Axial:
-      o << "axial,";
-      break;
-    case MWAWGraphicStyle::G_Linear:
-      o << "linear,";
-      break;
-    case MWAWGraphicStyle::G_Radial:
-      o << "radial,";
-      break;
-    case MWAWGraphicStyle::G_Rectangular:
-      o << "rectangular,";
-      break;
-    case MWAWGraphicStyle::G_Square:
-      o << "square,";
-      break;
-    case MWAWGraphicStyle::G_Ellipsoid:
-      o << "ellipsoid,";
-      break;
-    case MWAWGraphicStyle::G_None:
-#if !defined(__clang__)
-    default:
-#endif
-      break;
-    }
-    if (st.m_gradientAngle>0 || st.m_gradientAngle<0) o << "angle=" << st.m_gradientAngle << ",";
-    if (st.m_gradientStopList.size() >= 2) {
-      o << "stops=[";
-      for (auto const &grad : st.m_gradientStopList)
-        o << "[" << grad << "],";
-      o << "],";
-    }
-    if (st.m_gradientBorder>0) o << "border=" << st.m_gradientBorder*100 << "%,";
-    if (st.m_gradientPercentCenter != MWAWVec2f(0.5f,0.5f)) o << "center=" << st.m_gradientPercentCenter << ",";
-    if (st.m_gradientRadius<1) o << "radius=" << st.m_gradientRadius << ",";
-    o << "],";
-  }
+  if (st.hasGradient())
+    o << "grad=[" << st.m_gradient << "],";
+  if (st.hasHatch())
+    o << "hatch=[" << st.m_hatch << "],";
   if (st.hasShadow()) {
     o << "shadow=[";
     if (!st.m_shadowColor.isBlack())
