@@ -172,6 +172,40 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 5, MWAWDocument::MWAW_K_DRAW));
         return res;
       }
+      if (type=="VINF") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 5, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
+    }
+    else if (creator=="DAD6") {
+      if (type=="drw6" || type=="drwt") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 6, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+      if (type=="VINF") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 6, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
+    }
+    else if (creator=="DAD7") {
+      if (type=="drw7" || type=="drwt") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 7, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+      if (type=="VINF") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 7, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
+    }
+    else if (creator=="DAD8") {
+      if (type=="drw8" || type=="drwt") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 8, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+      if (type=="VINF") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 8, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
     }
     else if (creator=="Dc@P" || creator=="Dk@P") {
       if (type=="APPL") {
@@ -255,6 +289,12 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="LMAN") {
+      if (type=="TEXT") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 7, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
     else if (creator=="LWTE") {
       if (type=="TEXT" || type=="ttro") {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_LIGHTWAYTEXT,1));
@@ -746,14 +786,48 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     return res;
   }
 #endif
-  if (val[0]==0x200 && val[1]==0x80 && val[2]==0 && val[3]==0 && (val[4]>>8)<=8 && (val[4]&0xff)==0) {
-    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential Canvas 5 file\n"));
-    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 5, MWAWDocument::MWAW_K_DRAW));
+  // Canvas
+  if (val[0]==0x200 && val[1]==0x80) {
+    if (val[2]==0 && val[3]==0 && (val[4]>>8)<=8 && (val[4]&0xff)==0) {
+      MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential Canvas 5 file\n"));
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 5, MWAWDocument::MWAW_K_DRAW));
+    }
+    else { // 0 followed by compression mode
+      input->seek(9, librevenge::RVNG_SEEK_SET);
+      auto len=input->readULong(4);
+      if (len>=0x800 && len<=0x8000) { // block size
+        auto len1=input->readULong(4);
+        if (len1>0x800 && len1<=0x800c) {
+          MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential Canvas 6-8 file\n"));
+          res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 6, MWAWDocument::MWAW_K_DRAW));
+        }
+      }
+    }
   }
-  if (val[0]==0x100 && val[1]==0x8000 && (val[2]>=0&&val[2]<=8) && val[3]==0 && (val[4]>>8)==0) {
-    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential Canvas 5 win file\n"));
-    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 5, MWAWDocument::MWAW_K_DRAW));
+  if (val[0]==0x100 && val[1]==0x8000) {
+    if ((val[2]>=0&&val[2]<=8) && val[3]==0 && (val[4]>>8)==0) {
+      MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential Canvas 5 win file\n"));
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 5, MWAWDocument::MWAW_K_DRAW));
+    }
+    else {
+      input->setReadInverted(true);
+      input->seek(9, librevenge::RVNG_SEEK_SET);
+      auto len=input->readULong(4);
+      if (len>=0x800 && len<=0x8000) { // block size
+        auto len1=input->readULong(4);
+        if (len1>0x800 && len1<=0x800c) {
+          MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential Canvas 6-8 win file\n"));
+          res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 6, MWAWDocument::MWAW_K_DRAW));
+        }
+      }
+      input->setReadInverted(false);
+    }
   }
+  if (val[0]==0 && val[1]==1 && val[2]==0x4441 && val[3]==0x4435 && val[4]==0x5052) {
+    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential Canvas 5/6 image file\n"));
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 5, MWAWDocument::MWAW_K_PAINT));
+  }
+
   if (val[0]==2 && val[1]==0 && val[2]==2 && val[3]==0x262 && val[4]==0x262) {
     MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a MacDraft file\n"));
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAFT, 1, MWAWDocument::MWAW_K_DRAW));
