@@ -1411,14 +1411,14 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
 //////////////////////////////////////////////
 bool MoreText::readFonts(MWAWEntry const &entry)
 {
-  if (!entry.valid()) {
+  MWAWInputStreamPtr &input= m_parserState->m_input;
+  if (!entry.valid() || !input || !input->checkPosition(entry.end())) {
     MWAW_DEBUG_MSG(("MoreText::readFonts: the entry is bad\n"));
     return false;
   }
 
   long pos = entry.begin();
   long endPos = entry.end();
-  MWAWInputStreamPtr &input= m_parserState->m_input;
   libmwaw::DebugFile &ascFile = m_parserState->m_asciiFile;
   libmwaw::DebugStream f;
 
@@ -1871,9 +1871,9 @@ bool MoreText::readFont(MWAWEntry const &entry, std::string &fName, int &fId)
 {
   fName="";
   fId=-1;
-  if (entry.length() < 2)
-    return false;
   MWAWInputStreamPtr &input= m_parserState->m_input;
+  if (entry.length() < 2 || !input->checkPosition(entry.end()))
+    return false;
   long pos = entry.begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
@@ -1900,10 +1900,10 @@ bool MoreText::readFont(MWAWEntry const &entry, std::string &fName, int &fId)
 bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
 {
   level=MWAWListLevel();
-  if (entry.length()<22)
+  MWAWInputStreamPtr &input= m_parserState->m_input;
+  if (entry.length()<22 || !input || !input->checkPosition(entry.end()))
     return false;
 
-  MWAWInputStreamPtr &input= m_parserState->m_input;
   libmwaw::DebugStream f;
   long pos = entry.begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
