@@ -302,7 +302,19 @@ bool File::readFileInformation()
   }
   else if (m_fInfoCreator=="DAD5") {
     checkFInfoType("drw5","Canvas 5") || checkFInfoType("drwt","Canvas 5[template]") ||
-    checkFInfoType("Canvas 5");
+    checkFInfoType("VINF","Canvas 5[image]") || checkFInfoType("Canvas 5");
+  }
+  else if (m_fInfoCreator=="DAD6") {
+    checkFInfoType("drw6","Canvas 6") || checkFInfoType("drwt","Canvas 6[template]") ||
+    checkFInfoType("VINF","Canvas 6[image]") || checkFInfoType("Canvas 6");
+  }
+  else if (m_fInfoCreator=="DAD7") {
+    checkFInfoType("drw7","Canvas 7") || checkFInfoType("drwt","Canvas 7[template]") ||
+    checkFInfoType("VINF","Canvas 7[image]") || checkFInfoType("Canvas 7");
+  }
+  else if (m_fInfoCreator=="DAD8") {
+    checkFInfoType("drw8","Canvas 8") || checkFInfoType("drwt","Canvas 8[template]") ||
+    checkFInfoType("VINF","Canvas 8[image]") || checkFInfoType("Canvas 8");
   }
   else if (m_fInfoCreator=="DkmR") {
     checkFInfoType("TEXT","Basic text(created by DOCMaker)") || checkFInfoType("DOCMaker");
@@ -375,6 +387,9 @@ bool File::readFileInformation()
   else if (m_fInfoCreator=="LETR") {
     checkFInfoType("APPL","Take A Letter[auto]") || checkFInfoType("Take A Letter");
   }
+  else if (m_fInfoCreator=="LMAN") { // tutorial installed with canvas
+    checkFInfoType("TEXT","Canvas 7") || checkFInfoType("Canvas 7");
+  }
   else if (m_fInfoCreator=="LWTE") {
     checkFInfoType("TEXT","LightWayText") || checkFInfoType("MACR","LightWayText[MACR]") ||
     checkFInfoType("pref","LightWayText[Preferences]") ||
@@ -789,10 +804,46 @@ bool File::readDataInformation()
       return true;
     }
   }
-  if (val[0]==0x200 && val[1]==0x80 && val[2]==0 && val[3]==0 && (val[4]==0 || val[4]==0x800)) {
-    m_dataResult.push_back("Canvas 5");
+  if (val[0]==0 && val[1]==1 && val[2]==0x4441 && val[3]==0x4435 && val[4]==0x5052) {
+    m_dataResult.push_back("Canvas Image 5-7");
     return true;
   }
+  if (val[0]==0x200 && val[1]==0x80) {
+    if (val[2]==0 && val[3]==0 && (val[4]>>8)<=8 && (val[4]&0xff)==0) {
+      m_dataResult.push_back("Canvas 5[mac]");
+      return true;
+    }
+    input.seek(9, InputStream::SK_SET);
+    auto len=input.readU32();
+    if (len>=0x800 && len<=0x8000) { // block size
+      auto len1=input.readU32();
+      if (len1>0x800 && len1<=0x800c) {
+        m_dataResult.push_back("Canvas 6-8[mac]");
+        return true;
+      }
+    }
+  }
+  if (val[0]==0x100 && val[1]==0x8000) {
+    if ((val[2]>=0&&val[2]<=8) && val[3]==0 && (val[4]>>8)==0) {
+      m_dataResult.push_back("Canvas 5[windows]");
+      return true;
+    }
+
+    input.seek(9, InputStream::SK_SET);
+    unsigned len=0, decal=0;
+    for (int i=0; i<4; ++i, decal+=8)
+      len |= unsigned(input.readU8())<<decal;
+    if (len>=0x800 && len<=0x8000) { // block size
+      unsigned len1=0;
+      decal=0;
+      for (int i=0; i<4; ++i, decal+=8)
+        len1 |= unsigned(input.readU8())<<decal;
+      if (len1>0x800 && len1<=0x800c) {
+        m_dataResult.push_back("Canvas 6-8[windows]");
+        return true;
+      }
+    }
+  }
   if (val[0]==0xffd8 &&
       ((val[1]==0xffe0 && val[3]==0x4a46 && val[4] == 0x4946) ||
        (val[1]==0xffe1 && val[3]==0x4578 && val[4] == 0x6966) ||
